@@ -35,6 +35,9 @@
 #define GOWER 12
 #define ALTGOWER 13
 #define MIXED 14
+#define METRICMIXED 15
+
+#define EPS 1e-12
 
 /* Distance functions */
 
@@ -476,11 +479,22 @@ void xx_chisq_dist(double *x, int *nr, int *nc, double *d,
     }
 }
 
+
+/*
+ * Utility function
+ */
+static int allEqual(double x1, double x2)
+{
+  double dev;
+  dev = fabs(x1 - x2);
+  return (dev < EPS);
+}
+
 /*
  * Gower's coefficient for mixed data
  *
  * Should be called separately from the underlying R code,
- * not via xy_distance.
+ * not via xx_distance.
  *
  * vtype  : variable type
  *          1 == Symmetric Binary
@@ -490,11 +504,15 @@ void xx_chisq_dist(double *x, int *nr, int *nc, double *d,
  *          5 == Quantitative
  * weights: variable weights
  * R      : variable range (max - min)
+ * T      : dimensioned as x but with number of observations with
+ *          same rank as that observation. 0s everywhere other than
+ *          for ordinal variables as it's ignored elsewhere.
+ * Trange : vector of length nc with T range per variable
  *
  */
 double xx_MIXED(double *x, int nr, int nc, int i1, int i2, 
 		int *vtype, double *weights, double *R, 
-		double wsum)
+		double wsum, double *T, double *Trange)
 {
   double dist, dev;
   int count, j;
@@ -507,14 +525,14 @@ double xx_MIXED(double *x, int nr, int nc, int i1, int i2,
     if (R_FINITE(x[i1]) && R_FINITE(x[i2])) {
       // Symmetric binary
       if(vtype[j] == 1) {
-	dev = (x[i1] == x[i2]) ? 1 : 0;
+	dev = allEqual(x[i1], x[i2]) ? 1 : 0;
 	dist += dev * weights[j];
       }
       // Asymmetric binary
       if(vtype[j] == 2) {
 	if((x[i1] != 0) || (x[i2] != 0)) {
-	  // both x1 and x2 not zero for this variables
-	  dev = (x[i1] == x[i2]) ? 1 : 0;
+	  // either x1 and x2 not zero for this variable
+	  dev = allEqual(x[i1], x[i2]) ? 1 : 0;
 	  dist += dev * weights[j];
 	} else {
 	  /* set jth current weight to zero and do not
@@ -527,15 +545,18 @@ double xx_MIXED(double *x, int nr, int nc, int i1, int i2,
       }
       // Nominal
       if(vtype[j] == 3) {
-	dev = (x[i1] == x[i2]) ? 1 : 0;
+	dev = allEqual(x[i1], x[i2]) ? 1 : 0;
 	dist += dev * weights[j];
       }
-      // Ordinal
+      // Ordinal (Eqns 2a and 2b in Podani 1999); x converted to ranks in R-land
       if(vtype[j] == 4) {
-	/* ordinal data currently handled as Nominal */
-	dev = (x[i1] == x[i2]) ? 1 : 0;
+	if(allEqual(x[i1], x[i2])) {	/* Eqn 2a */
+	  dev = 1;
+	} else {		/* Eqn 2b */
+	  // T has already had the (T - 1) / 2 adjustment done in
+	  dev = 1 - ((fabs(x[i1] - x[i2]) - T[i1] - T[i2]) / (R[j] - Trange[j]));
+	}
 	dist += dev * weights[j];
-	break;
       }
       // Quantitative
       if(vtype[j] == 5) {
@@ -553,25 +574,9 @@ double xx_MIXED(double *x, int nr, int nc, int i1, int i2,
   return 1 - (dist / wsum);
 }
 
-/*
-double xx_calcTI(double *x, double *y, int nr1, int nr2, int nc, int i1, int i2)
-{
-int k;
-double ti;
-
-ti = 0.0;
-
-for (k=0; k<nc; k++) {
-ti += (x[i1] == y[i2]) ? 1.0 : 0.0;
-i1 += nr1;
-i2 += nr2;
-}
-return ti;
-}
-*/
-
 void xx_mixed(double *x, int *nr, int *nc, double *d, 
-	      int *diag, int *vtype, double *weights, double *R)
+	      int *diag, int *vtype, double *weights, double *R,
+	      double *T, double *Trange)
 {
     int dc, i, j, k, ij;
     double wsum;
@@ -589,7 +594,114 @@ void xx_mixed(double *x, int *nr, int *nc, double *d,
     for(j=0; j < *nr; j++) {
         for(i=j+dc; i < *nr; i++) {
 	  d[ij++] = xx_MIXED(x, *nr, *nc, i, j, vtype,
-			     weights, R, wsum);
+			     weights, R, wsum, T, Trange);
 	}
     }
 }
+
+/*
+ * This is the *metric* version of Podanis' modification to Gower's mixed
+ * coefficient
+ *
+ *
+ * Gower's coefficient for mixed data
+ *
+ * Should be called separately from the underlying R code,
+ * not via xx_distance.
+ *
+ * vtype  : variable type
+ *          1 == Symmetric Binary
+ *          2 == Asymmetric Binary
+ *          3 == Nominal (class/factor)
+ *          4 == Ordinal (ordered factor)
+ *          5 == Quantitative
+ * weights: variable weights
+ * R      : variable range (max - min)
+ *
+ */
+
+double xx_METRICMIXED(double *x, int nr, int nc, int i1, int i2, 
+		      int *vtype, double *weights, double *R, 
+		      double wsum)
+{
+  double dist, dev;
+  int count, j;
+
+  count = 0;
+  dist = 0.0;
+  wsum = 0.0;
+  
+  for (j=0; j<nc; j++) {
+    if (R_FINITE(x[i1]) && R_FINITE(x[i2])) {
+      // Symmetric binary
+      if(vtype[j] == 1) {
+	dev = allEqual(x[i1], x[i2]) ? 1 : 0;
+	dist += dev * weights[j];
+      }
+      // Asymmetric binary
+      if(vtype[j] == 2) {
+	if((x[i1] != 0) || (x[i2] != 0)) {
+	  // both x1 and x2 not zero for this variables
+	  dev = allEqual(x[i1], x[i2]) ? 1 : 0;
+	  dist += dev * weights[j];
+	} else {
+	  /* set jth current weight to zero and do not
+	     increment dist as ignoring double zero
+	     We actually subtract the weight as it gets added
+	     later on.
+	  */
+	  wsum -= weights[j];
+	}
+      }
+      // Nominal
+      if(vtype[j] == 3) {
+	dev = allEqual(x[i1], x[i2]) ? 1 : 0;
+	dist += dev * weights[j];
+      }
+      // Ordinal
+      if(vtype[j] == 4) {
+	/* ordinal data handled as metric version of Podani
+	 * which is quantitatively using ranks
+         */
+	dev = 1 - (fabs(x[i1] - x[i2]) / R[j]);
+	dist += dev * weights[j];
+      }
+      // Quantitative
+      if(vtype[j] == 5) {
+	dev = 1 - (fabs(x[i1] - x[i2]) / R[j]);
+	dist += dev * weights[j];
+      }
+      count++;
+      // only summing weights for non-NA comparisons
+      wsum += weights[j];
+    }
+    i1 += nr;
+    i2 += nr;
+  }
+  if (count == 0) return NA_REAL;
+  return 1 - (dist / wsum);
+}
+
+void xx_metric_mixed(double *x, int *nr, int *nc, double *d, 
+		     int *diag, int *vtype, double *weights, double *R)
+{
+  int dc, i, j, k, ij;
+  double wsum;
+  
+  wsum = 0.0;
+  
+  ij = 0;
+  
+  for(k=0; k <*nc; k++) {
+    wsum += weights[k];
+  }
+  
+  dc = (*diag) ? 0 : 1;
+  
+  for(j=0; j < *nr; j++) {
+    for(i=j+dc; i < *nr; i++) {
+      d[ij++] = xx_METRICMIXED(x, *nr, *nc, i, j, vtype,
+			       weights, R, wsum);
+    }
+  }
+}
